@@ -265,7 +265,7 @@ from time import sleep
 import aiohttp
 from twitchAPI.twitch import Twitch
 from twitchAPI.object import TwitchUser
-from twitchAPI.helper import TWITCH_CHAT_URL, first
+from twitchAPI.helper import TWITCH_CHAT_URL, first, RateLimitBucket, RATE_LIMIT_SIZES
 from twitchAPI.types import ChatRoom, TwitchBackendException, AuthType, AuthScope, ChatEvent, MissingScopeException, UnauthorizedException
 
 from typing import List, Optional, Union, Callable, Dict, Awaitable, Any
@@ -428,6 +428,8 @@ class ChatMessage(EventData):
 
     async def reply(self, text: str):
         """Reply to this message"""
+        bucket = self.chat._get_message_bucket(self._parsed['command']['channel'][1:])
+        await bucket.put()
         await self.chat.send_raw_irc_message(f'@reply-parent-msg-id={self.id} PRIVMSG #{self.room.name} :{text}')
 
 
@@ -518,7 +520,12 @@ EVENT_CALLBACK_TYPE = Callable[[Any], Awaitable[None]]
 class Chat:
     """The chat bot instance"""
 
-    def __init__(self, twitch: Twitch, connection_url: Optional[str] = None):
+    def __init__(self, twitch: Twitch, connection_url: Optional[str] = None, is_verified_bot: bool = False):
+        """
+        :param twitch: A Authenticated twitch instance
+        :param connection_url: alternative connection url |default|:code:`None`
+        :param is_verified_bot: set to true if your bot is verified by twitch |default|:code:`False`
+        """
         self.logger: Logger = getLogger('twitchAPI.chat')
         """The logger used for Chat related log messages"""
         self._prefix: str = "!"
@@ -537,6 +544,9 @@ class Chat:
         self.__socket_loop = None
         self.__startup_complete: bool = False
         self.__tasks = None
+        self._ready = False
+        self._send_buckets = {}
+        self._join_bucket = RateLimitBucket(10, 2000 if is_verified_bot else 20, 'channel_join', self.logger)
         self.__waiting_for_pong: bool = False
         self._event_handler = {}
         self._command_handler = {}
@@ -545,6 +555,7 @@ class Chat:
         self._room_leave_locks = []
         self._closing: bool = False
         self.join_timeout: int = 10
+        self._mod_status_cache = {}
         """Time in seconds till a channel join attempt times out"""
 
     def __await__(self):
@@ -738,6 +749,7 @@ class Chat:
             raise UnauthorizedException('CHAT_READ authscope is required to run a chat bot')
         self.__startup_complete = False
         self._closing = False
+        self._ready = False
         self.__socket_thread = threading.Thread(target=self.__run_socket)
         self.__running = True
         self.__socket_thread.start()
@@ -757,6 +769,7 @@ class Chat:
         self.logger.debug('stopping chat...')
         self.__startup_complete = False
         self.__running = False
+        self._ready = False
         f = asyncio.run_coroutine_threadsafe(self._stop(), self.__socket_loop)
         f.result()
 
@@ -817,7 +830,7 @@ class Chat:
             traceback.print_exception(type(e), e, e.__traceback__, file=sys.stderr)
 
     async def _send_message(self, message: str):
-        self.logger.debug(f'Sending message "{message}"')
+        self.logger.debug(f'> "{message}"')
         await self.__connection.send_str(message)
 
     async def __task_receive(self):
@@ -834,7 +847,8 @@ class Chat:
                 'PART': self._handle_part,
                 'NOTICE': self._handle_notice,
                 'CLEARCHAT': self._handle_clear_chat,
-                'WHISPER': self._handle_whisper
+                'WHISPER': self._handle_whisper,
+                'RECONNECT': self._handle_reconnect
             }
             while not self.__connection.closed:
                 message = await self.__connection.receive()
@@ -862,6 +876,11 @@ class Chat:
             # print('we are closing down!')
             return
 
+    async def _handle_reconnect(self, parsed: dict):
+        self.logger.info('got reconnect request...')
+        await self.__connect(is_startup=False)
+        self.logger.info('reconnect completed')
+
     async def _handle_whisper(self, parsed: dict):
         e = WhisperEvent(self, parsed)
         for handler in self._event_handler.get(ChatEvent.WHISPER, []):
@@ -963,12 +982,17 @@ class Chat:
     async def _handle_ready(self, parsed: dict):
         self.logger.debug('got ready event')
         dat = EventData(self)
+        self._ready = True
         for h in self._event_handler.get(ChatEvent.READY, []):
             t = asyncio.ensure_future(h(dat))
             t.add_done_callback(self._task_callback)
 
     async def _handle_msg(self, parsed: dict):
         self.logger.debug('got new message, call handler')
+        # get own state based on badges
+        if parsed['source']['nick'][1:] == self.username:
+            self._mod_status_cache[parsed['command']['channel'][1:]] = 'mod' if parsed['tags']['mod'] == '1' or \
+                                                                                parsed['tags']['badges'].get('broadcaster') is not None else 'user'
         if parsed['command'].get('bot_command') is not None:
             command_name = parsed['command'].get('bot_command').lower()
             handler = self._command_handler.get(command_name)
@@ -989,6 +1013,16 @@ class Chat:
         await self._send_message(f'NICK {self.username}')
         self.__startup_complete = True
 
+    def _get_message_bucket(self, channel) -> RateLimitBucket:
+        bucket = self._send_buckets.get(channel)
+        if bucket is None:
+            bucket = RateLimitBucket(30, 20, channel, self.logger)
+            self._send_buckets[channel] = bucket
+        target_size = RATE_LIMIT_SIZES[self._mod_status_cache.get(channel, 'user')]
+        if bucket.bucket_size != target_size:
+            bucket.bucket_size = target_size
+        return bucket
+
     ##################################################################################################################################################
     # user functions
     ##################################################################################################################################################
@@ -1044,6 +1078,16 @@ class Chat:
         self._event_handler[event].remove(handler)
         return True
 
+    def is_connected(self) -> bool:
+        """Returns your current connection status."""
+        if self.__connection is None:
+            return False
+        return not self.__connection.closed
+
+    def is_ready(self) -> bool:
+        """Returns True if the chat bot is ready to join channels and/or receive events"""
+        return self._ready
+
     async def join_room(self, chat_rooms: Union[List[str], str]):
         """ join one or more chat rooms\n
         Will only exit once all given chat rooms where successfully joined or :const:`twitchAPI.chat.Chat.join_timeout` run out.
@@ -1057,6 +1101,7 @@ class Chat:
         target = [c[1:].lower() if c[0] == '#' else c.lower() for c in chat_rooms]
         for r in target:
             self._room_join_locks.append(r)
+        await self._join_bucket.put()
         await self._send_message(f'JOIN {room_str}')
         # wait for us to join all rooms
         timeout = datetime.datetime.now() + datetime.timedelta(seconds=self.join_timeout)
@@ -1071,7 +1116,12 @@ class Chat:
         """Send a raw IRC message
 
         :param message: the message to send
+        :raises ValueError: if bot is not ready
         """
+        if not self.is_ready():
+            raise ValueError('can\'t send message: bot not ready')
+        while not self.is_connected():
+            await asyncio.sleep(0.1)
         if message is None or len(message) == 0:
             raise ValueError('message must be a non empty string')
         await self._send_message(message)
@@ -1085,7 +1135,12 @@ class Chat:
             This can either be a instance of :const:`~twitchAPI.types.ChatRoom` or a string with the room name (either with leading # or without)
         :param text: The text you want to send
         :raises ValueError: if message is empty or room is not given
+        :raises ValueError: if bot is not ready
         """
+        if not self.is_ready():
+            raise ValueError('can\'t send message: bot not ready')
+        while not self.is_connected():
+            await asyncio.sleep(0.1)
         if isinstance(room, ChatRoom):
             room = room.name
         if room is None or len(room) == 0:
@@ -1094,6 +1149,8 @@ class Chat:
             raise ValueError('you can\'t send a empty message')
         if room[0] != '#':
             room = f'#{room}'
+        bucket = self._get_message_bucket(room[1:])
+        await bucket.put()
         await self._send_message(f'PRIVMSG {room} :{text}')
 
     async def leave_room(self, chat_rooms: Union[List[str], str]):
